﻿QuestProgressSound = LibStub( "AceAddon-3.0"):NewAddon( "QuestProgressSound", "AceConsole-3.0", "AceEvent-3.0", "LibSink-2.0")
local Quixote = LibStub("LibQuixote-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("QuestProgressSound")

local MAJOR_VERSION = "1.3"

local sounds = {
	soundObjectiveUpdate = {
		"",
		"Sound\\Creature\\Peon\\PeonReady1.wav",
		"Sound\\Creature\\Peasant\\PeasantWhat3.wav",
	},
	soundQuestComplete = {
		"",
		"Sound\\Creature\\Peon\\PeonBuildingComplete1.wav",
		"Interface\\Addons\\QuestProgressSound\\Sound\\Peasant_work_done.mp3",
	},
	soundparty = {
		"",
		"Interface\\Addons\\QuestProgressSound\\Sound\\ding.mp3",
		"Interface\\Addons\\QuestProgressSound\\Sound\\tada.mp3",
		"Interface\\Addons\\QuestProgressSound\\Sound\\Alert.mp3",
		"Interface\\Addons\\QuestProgressSound\\Sound\\Info.mp3",
	},
}

local soundnames = {
	soundObjectiveUpdate = {
		"None",
		"Peon - Ready",
		"Peasant - More Work",
	},
	soundQuestComplete = {
		"None",
		"Peon - Complete",
		"Peasant - Job's done",
	},
	soundparty = {
		"None",
		"Ding",
		"Tada",
		"Alert",
		"Info",
	},
}

local STARTCOLOR = "|cFFD82619"
local MIDDLECOLOUR = "|cFFFFFF00";
local ENDCOLOR = "|cFF00FF19"
local TEXTCOLOR = "|cFFBFBFFF"

local options = {
	name = "QuestProgressSound",
	handler = QuestProgressSound,
	type = "group",
	childGroups = "tab",
	get = "getValue",
	set = "setValue",
	args = {
		player = {
			name = L["Player Sounds"],
			type = "group",
			args = {
				enableSound = {
					name = L["Enable Sound"],
					desc = L["Enables / disables sound"],
					type = "toggle",
				},
				soundObjectiveUpdate = {
					name = L["Object Sound"],
					desc = L["Sound played when Part of Quest completed"],
					type = "select",
					values = soundnames.soundObjectiveUpdate,
					set = "setSound",
				},
				soundQuestComplete = {
					name = L["Quest Complete Sound"],
					desc = L["Sound played when all quest objects completed"],
					type = "select",
					values = soundnames.soundQuestComplete,
					set = "setSound",
				},
			},
		},
		party = {
			name = L["Party Sounds"],
			type = "group",
			args = {
				enablePartyQuestProgress = {
					name = L["Enable Party Progress Sound"],
					desc = L["Plays defined sound when someone in your party fulfilled a part of a quest"],
					type = "toggle",
				},
				enablePartyQuestComplete = {
					name = L["Enable Party Quest Complete"],
					desc = L["Plays defined sound when someone in your party completes a quest"],
					type = "toggle",
				},
				soundpartyprogress = {
					name = L["Party Progress Sound"],
					desc = L["Sound played when Part of Quest completed by someone in your party"],
					type = "select",
					values = soundnames.soundparty,
					set = "setSoundParty",
				},
				soundpartycomplete = {
					name = L["Party Quest Complete Sound"],
					desc = L["Sound played when all quest objects completed by someone in your party"],
					type = "select",
					values = soundnames.soundparty,
					set = "setSoundParty",
				},
			},
		},
		textnotify = {
			name = "Visual Party Progress",
			type = "group",
			args = {
				enableOutput = {
					name = L["Enable Output"],
					desc = L["Shows progress of party members"],
					type = "toggle",
				},
				output = QuestProgressSound:GetSinkAce3OptionsDataTable(),
			},
		},
	},
}

local defaults = {
	profile = {
		enableSound = true,
		enableOutput = false,
		sinkoptions = {},
		soundObjectiveUpdate = 1,
		soundQuestComplete = 1,
		enablePartyQuestProgress = false,
		enablePartyQuestComplete = false,
		soundpartyprogress = 1,
		soundpartycomplete = 1,
	}
}

function QuestProgressSound:getValue(info)
	return self.db.profile[info[#info]]
end

function QuestProgressSound:setValue(info,value)
	self.db.profile[info[#info]] = value
end

function QuestProgressSound:setSound(info,value)
	self.db.profile[info[#info]] = value
	if( sounds[info[#info]][value]~="") then
		PlaySoundFile(sounds[info[#info]][value])
	end
end

function QuestProgressSound:setSoundParty(info,value)
	self.db.profile[info[#info]] = value
	if( sounds.soundparty[value]~="") then
		PlaySoundFile(sounds.soundparty[value])
	end
end

function QuestProgressSound:OnInitialize()
	-- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("QuestProgressSoundDB", defaults, "Default")
	
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	self:SetSinkStorage(self.db.profile.sinkoptions)
	
	options.args.textnotify.args.output.disabled = function() return not self.db.profile.enableOutput end
	options.args.party.args.soundpartyprogress.disabled = function() return not self.db.profile.enablePartyQuestProgress end
	options.args.party.args.soundpartycomplete.disabled = function() return not self.db.profile.enablePartyQuestComplete end
	options.args.player.args.soundObjectiveUpdate.disabled = function() return not self.db.profile.enableSound end
	options.args.player.args.soundQuestComplete.disabled = function() return not self.db.profile.enableSound end
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("QuestProgressSound", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuestProgressSound","QuestProgressSound")
	self:RegisterChatCommand("qps", "ChatCommand")
end

function QuestProgressSound:OnEnable()
	-- Called when the addon is enabled
	self:Print(L["Quest Progress Sounds loaded"])
	Quixote.RegisterCallback(self, "Objective_Update")
	Quixote.RegisterCallback(self, "Quest_Complete")
	Quixote.RegisterCallback(self, "Party_Objective_Update")
	Quixote.RegisterCallback(self, "Party_Quest_Complete")
end

function QuestProgressSound:Objective_Update(eventname, questtitle,uid,desc,old,currentobjects,neededobjects,type)
	local complete = select(7,Quixote:GetQuestByUid(uid))
	if (self.db.profile.enableSound) and (complete == nil) and (currentobjects==neededobjects) and (sounds.soundObjectiveUpdate[self.db.profile.soundObjectiveUpdate]~="") then
		PlaySoundFile(sounds.soundObjectiveUpdate[self.db.profile.soundObjectiveUpdate])
	end
end

function QuestProgressSound:Quest_Complete()
	if (self.db.profile.enableSound) and (sounds.soundQuestComplete[self.db.profile.soundQuestComplete]~="") then
		PlaySoundFile(sounds.soundQuestComplete[self.db.profile.soundQuestComplete])
	end
end

function QuestProgressSound:Party_Objective_Update(event, sender, uid, questtitle, objective, numGot, numNeeded)
	if (self.db.profile.enableOutput) then
		self:Pour(L["Questprogress of "]..self:getClassColor(sender)..sender..TEXTCOLOR..": "..questtitle.." "..self:GetColourString(numGot,numNeeded)..numGot.."/"..numNeeded..TEXTCOLOR.." "..objective)
	end
	if (self.db.profile.enablePartyQuestProgress) and (numGot==numNeeded) and (sounds.soundparty[self.db.profile.soundpartyprogress]~="") then
		PlaySoundFile(sounds.soundparty[self.db.profile.soundpartyprogress])
	end
end

function QuestProgressSound:Party_Quest_Complete(event, sender, uid, questtitle)
	if (self.db.profile.enableOutput) then
		self:Pour(sender.." "..ENDCOLOR..L["completed"]..TEXTCOLOR.." "..questtitle)
	end
	if (self.db.profile.enablePartyQuestComplete) and (sounds.soundparty[self.db.profile.soundpartycomplete]~="") then
		PlaySoundFile(sounds.soundparty[self.db.profile.soundpartycomplete])
	end
end

function QuestProgressSound:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function QuestProgressSound:TextToRGB(strColour)
	local i = 3;
	local iAlpha = tonumber(string.sub(strColour, i, i + 1), 16);
	local iRed = tonumber(string.sub(strColour, i + 2, i + 3), 16);
	local iGreen = tonumber(string.sub(strColour, i + 4, i + 5), 16);
	local iBlue = tonumber(string.sub(strColour, i + 6, i + 7), 16);
	iAlpha = iAlpha / 255;
	iRed = iRed / 255;
	iGreen = iGreen / 255;
	iBlue = iBlue / 255;
	return iRed, iGreen, iBlue, iAlpha;
end

function QuestProgressSound:RGBtoText(iRed, iGreen, iBlue, iAlpha)
	local strColour;
	iAlpha = floor(iAlpha * 255);
	iRed = floor(iRed * 255);
	iGreen = floor(iGreen * 255);
	iBlue = floor(iBlue * 255);
	strColour = format("|c%2x%2x%2x%2x", iAlpha, iRed, iGreen, iBlue);
	return strColour;
end

function QuestProgressSound:GetColourString(NumItems, NumNeeded)
	local colour = {a = 1.0, r = 1.0, g = 1.0, b = 1.0};
	local colourInitial = {a = 1.0, r = 1.0, g = 1.0, b = 1.0};
	local colourMid = {a = 1.0, r = 1.0, g = 1.0, b = 1.0};
	local colourComplete = {a = 1.0, r = 1.0, g = 1.0, b = 1.0};
	colourInitial.r, colourInitial.g, colourInitial.b, colourInitial.a = self:TextToRGB(STARTCOLOR);
	colourMid.r, colourMid.g, colourMid.b, colourMid.a = self:TextToRGB(MIDDLECOLOUR);
	colourComplete.r, colourComplete.g, colourComplete.b, colourComplete.a = self:TextToRGB(ENDCOLOR);

	local colourDelta1 = {
		a = (colourMid.a - colourInitial.a),
		r = (colourMid.r - colourInitial.r),
		g = (colourMid.g - colourInitial.g),
		b = (colourMid.b - colourInitial.b)
	};
	local colourDelta2 = {
		a = (colourComplete.a - colourMid.a),
		r = (colourComplete.r - colourMid.r),
		g = (colourComplete.g - colourMid.g),
		b = (colourComplete.b - colourMid.b)
	};
	if ((NumItems / NumNeeded) < 0.5) then
		colour.r = colourInitial.r + ((NumItems / (NumNeeded / 2)) * colourDelta1.r);
		colour.g = colourInitial.g + ((NumItems / (NumNeeded / 2)) * colourDelta1.g);
		colour.b = colourInitial.b + ((NumItems / (NumNeeded / 2)) * colourDelta1.b);
	else
		colour.r = colourMid.r + (((NumItems - (NumNeeded / 2)) / (NumNeeded / 2)) * colourDelta2.r);
		colour.g = colourMid.g + (((NumItems - (NumNeeded / 2)) / (NumNeeded / 2)) * colourDelta2.g);
		colour.b = colourMid.b + (((NumItems - (NumNeeded / 2)) / (NumNeeded / 2)) * colourDelta2.b);
	end

	if(colour.r > 1.0) then colour.r = 1.0; end
	if(colour.g > 1.0) then colour.g = 1.0; end
	if(colour.b > 1.0) then colour.b = 1.0; end
	if(colour.r < 0.0) then colour.r = 0.0; end
	if(colour.g < 0.0) then colour.g = 0.0; end
	if(colour.b < 0.0) then colour.b = 0.0; end

	return self:RGBtoText(colour.r, colour.g, colour.b, colour.a);
end

function QuestProgressSound:getClassColor( sender )
	local localizedclass, englishclass = UnitClass(sender)
	local c = RAID_CLASS_COLORS[englishclass]
	if (c) then
		c = format("|c00%02X%02X%02X", 255 * c.r, 255 * c.g, 255 * c.b)
	else
		c = "|c00808080"
	end
	return c
end